@@ -2,17 +2,18 @@
 
 from __future__ import annotations
 
-import re
 import asyncio
 from datetime import datetime
-from typing import Any, Dict, Optional, List
-from uuid import UUID
+from typing import Any, Dict, Optional, List, TYPE_CHECKING
 
-from pydantic import BaseModel, Field, validator
+from pydantic import BaseModel, Field, validator, field_serializer
 
 # Import the shared VerifierFunction type that works for both async and sync
 from fleet.types import VerifierFunction
 
+if TYPE_CHECKING:
+    from fleet.models import VerifiersExecuteResponse
+
 
 class Task(BaseModel):
     """A task model representing a single task in the Fleet system."""
@@ -27,7 +28,9 @@ class Task(BaseModel):
     version: Optional[str] = Field(None, description="Task version")
     verifier_func: Optional[str] = Field(None, description="Verifier function code")
     verifier: Optional[Any] = Field(
-        None, description="Verifier function with decorator (async or sync)"
+        None,
+        description="Verifier function with decorator (async or sync)",
+        exclude=True,
     )
     verifier_id: Optional[str] = Field(None, description="Verifier identifier")
     verifier_sha: Optional[str] = Field(None, description="Verifier SHA256 hash")
@@ -44,10 +47,15 @@ class Task(BaseModel):
         """Set created_at to current time if not provided."""
         return v or datetime.now()
 
+    @field_serializer("created_at")
+    def serialize_created_at(self, dt: Optional[datetime], _info):
+        """Serialize datetime to ISO format string."""
+        return dt.isoformat() if dt else None
+
     @property
     def env_key(self) -> str:
         """Get the environment key combining env_id and version."""
-        if self.version and self.version != "None":
+        if self.version and self.version != "None" and ":" not in self.env_id:
             return f"{self.env_id}:{self.version}"
         return self.env_id
 
@@ -66,21 +74,21 @@ class Task(BaseModel):
         For sync environments, calls the sync verifier directly.
         For async verifiers, automatically runs them with asyncio.run().
         """
+        # If verifier doesn't exist but verifier_func does, rebuild it
+        if not self.verifier and self.verifier_func:
+            self._rebuild_verifier()
+
         if self.verifier:
             import inspect
 
             # Check if verifier has remote method (for decorated verifiers)
-            if hasattr(self.verifier, "remote"):
-                result = self.verifier.remote(env, *args, **kwargs)
-            else:
-                # For verifiers created from string, call directly
-                result = self.verifier(env, *args, **kwargs)
+            result = self.verifier.remote(env, *args, **kwargs)
 
             # If the result is a coroutine, we need to run it
             if inspect.iscoroutine(result):
                 # Check if we're already in an event loop
                 try:
-                    loop = asyncio.get_running_loop()
+                    asyncio.get_running_loop()
                     # We're in an async context, can't use asyncio.run()
                     raise RuntimeError(
                         "Cannot run async verifier in sync mode while event loop is running. "
@@ -100,6 +108,10 @@ class Task(BaseModel):
         For async environments, awaits the async verifier.
         Works with both sync and async verifiers in async contexts.
         """
+        # If verifier doesn't exist but verifier_func does, rebuild it
+        if not self.verifier and self.verifier_func:
+            self._rebuild_verifier()
+
         if self.verifier:
             result = self.verifier.remote(*args, **kwargs)
             # If it's a coroutine, await it
@@ -112,6 +124,70 @@ class Task(BaseModel):
         else:
             raise ValueError("No verifier function found for this task")
 
+    def verify_detailed(self, env, *args, **kwargs) -> "VerifiersExecuteResponse":
+        """Verify the task and return the full execute response model.
+
+        For sync environments, calls the sync verifier directly.
+        For async verifiers, automatically runs them with asyncio.run().
+        """
+        # If verifier doesn't exist but verifier_func does, rebuild it
+        if not self.verifier and self.verifier_func:
+            self._rebuild_verifier()
+
+        if self.verifier:
+            import inspect
+
+            # Check if verifier has remote_with_response method (for decorated verifiers)
+            result = self.verifier.remote_with_response(env, *args, **kwargs)
+
+            # If the result is a coroutine, we need to run it
+            if inspect.iscoroutine(result):
+                # Check if we're already in an event loop
+                try:
+                    asyncio.get_running_loop()
+                    # We're in an async context, can't use asyncio.run()
+                    raise RuntimeError(
+                        "Cannot run async verifier in sync mode while event loop is running. "
+                        "Use await task.verify_detailed_async() instead."
+                    )
+                except RuntimeError:
+                    # No event loop running, safe to use asyncio.run()
+                    return asyncio.run(result)
+            else:
+                return result
+        else:
+            raise ValueError("No verifier function found for this task")
+
+    def verify_detailed_async(self, *args, **kwargs) -> "VerifiersExecuteResponse":
+        """Verify the task and return the full execute response model (async version).
+
+        For async environments, returns a coroutine that when awaited returns the response.
+        Works with both sync and async verifiers in async contexts.
+        """
+        # If verifier doesn't exist but verifier_func does, rebuild it
+        if not self.verifier and self.verifier_func:
+            self._rebuild_verifier()
+
+        if self.verifier:
+            result = self.verifier.remote_with_response(*args, **kwargs)
+            # Return the result (could be a coroutine or a value)
+            return result
+        else:
+            raise ValueError("No verifier function found for this task")
+
+    def _rebuild_verifier(self):
+        """Rebuild the verifier from verifier_func string if it exists."""
+        if self.verifier_func:
+            # Use the same logic as in verifier_from_string
+            verifier_id = self.verifier_id or self.key
+            verifier = verifier_from_string(
+                verifier_func=self.verifier_func,
+                verifier_id=verifier_id,
+                verifier_key=self.key,
+                sha256=self.verifier_sha or "",
+            )
+            self.verifier = verifier
+
     def make_env(self, region: Optional[str] = None):
         """Create an environment instance for this task's environment.
 
@@ -141,7 +217,7 @@ def verifier_from_string(
     """
     try:
         import inspect
-        from .verifiers import verifier, SyncVerifierFunction
+        from .verifiers import SyncVerifierFunction
         from .verifiers.code import TASK_SUCCESSFUL_SCORE, TASK_FAILED_SCORE
         from .verifiers.db import IgnoreConfig
 
@@ -172,36 +248,13 @@ def verifier_from_string(
         if func_obj is None:
             raise ValueError("No function found in verifier code")
 
-        # Create a wrapper function that provides the necessary globals
-        def wrapped_verifier(env, *args, **kwargs):
-            # Set up globals for the function execution
-            func_globals = (
-                func_obj.__globals__.copy() if hasattr(func_obj, "__globals__") else {}
-            )
-            func_globals.update(
-                {
-                    "TASK_SUCCESSFUL_SCORE": TASK_SUCCESSFUL_SCORE,
-                    "TASK_FAILED_SCORE": TASK_FAILED_SCORE,
-                    "IgnoreConfig": IgnoreConfig,
-                }
-            )
-
-            # Create a new function with the updated globals
-            import types
-
-            new_func = types.FunctionType(
-                func_obj.__code__,
-                func_globals,
-                func_obj.__name__,
-                func_obj.__defaults__,
-                func_obj.__closure__,
-            )
-
-            return new_func(env, *args, **kwargs)
-
-        # Create an AsyncVerifierFunction instance with the wrapped function
+        # Create an SyncVerifierFunction instance with raw code
         verifier_instance = SyncVerifierFunction(
-            wrapped_verifier, verifier_key, verifier_id
+            func=func_obj,
+            key=verifier_key,
+            verifier_id=verifier_id,
+            sha256=sha256,
+            raw_code=verifier_func,
         )
 
         # Store additional metadata
@@ -214,11 +267,24 @@ def verifier_from_string(
         raise ValueError(f"Failed to create verifier from string: {e}")
 
 
+def load_tasks_from_file(filename: str) -> List[Task]:
+    """Load tasks from a JSON file.
+
+    Example:
+        tasks = fleet.load_tasks_from_file("my_tasks.json")
+    """
+    from .global_client import get_client
+
+    client = get_client()
+    return client.load_tasks_from_file(filename)
+
+
 def load_tasks(
     env_key: Optional[str] = None,
     keys: Optional[List[str]] = None,
     version: Optional[str] = None,
     team_id: Optional[str] = None,
+    project_key: Optional[str] = None,
 ) -> List[Task]:
     """Convenience function to load tasks with optional filtering.
 
@@ -238,7 +304,11 @@ def load_tasks(
 
     client = get_client()
     return client.load_tasks(
-        env_key=env_key, keys=keys, version=version, team_id=team_id
+        env_key=env_key,
+        keys=keys,
+        version=version,
+        team_id=team_id,
+        project_key=project_key,
     )
 
 
@@ -260,9 +330,49 @@ def update_task(
         response = fleet.update_task("my-task", verifier_code="def verify(env): return True")
     """
     from .global_client import get_client
-    from .models import TaskResponse
 
     client = get_client()
     return client.update_task(
         task_key=task_key, prompt=prompt, verifier_code=verifier_code
     )
+
+
+def import_task(task: Task, project_key: Optional[str] = None):
+    """Convenience function to import a single task.
+
+    Args:
+        task: Task object to import
+        project_key: Optional project key to associate with the task
+
+    Returns:
+        Response from the API, or None if the import failed
+
+    Examples:
+        task = fleet.Task(key="my-task", prompt="Do something", env_id="my-env")
+        response = fleet.import_task(task)
+        response = fleet.import_task(task, project_key="my-project")
+    """
+    from .global_client import get_client
+
+    client = get_client()
+    return client.import_single_task(task, project_key=project_key)
+
+
+def import_tasks(filename: str, project_key: Optional[str] = None):
+    """Convenience function to import tasks from a JSON file.
+
+    Args:
+        filename: Path to the JSON file of Task objects to import
+        project_key: Optional project key to associate with the tasks
+
+    Returns:
+        List of responses from the API for successfully imported tasks
+
+    Examples:
+        responses = fleet.import_tasks("tasks.json")
+        responses = fleet.import_tasks("tasks.json", project_key="my-project")
+    """
+    from .global_client import get_client
+
+    client = get_client()
+    return client.import_tasks(filename, project_key=project_key)
